/*
 ****************************************************************
 *
 * Copyright (C) 2007 Atsushi Umeshima <pcamfm@gmail.com>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 ****************************************************************
 */

#include <stdio.h>
#include <string.h>
#include <getopt.h>
#include <sys/time.h>
#include <sys/types.h>
#include <unistd.h>

#include "amfm.h"


// Defines
#define NAME "amfm"
#define VERSION "2.0.1"
#define FCODE(f)    (f-7600)/5
#define ACODE(f)    (f-531)/9
#define FMFREQ_MIN 7600
#define FMFREQ_MAX 10800
#define FMFREQ_STEP 5
#define AMFREQ_MIN 531
#define AMFREQ_MAX 1602
#define AMFREQ_STEP 9
#define VOLUME_MIN 0
#define VOLUME_MAX 100
#define INT_SHORT 10000
#define INT_LONG  50000
#define OFF_SLEEP_USEC 50000
#define ON_SLEEP_USEC 100000
#define TUNE_SLEEP_USEC 50000


// Global Variables
char *myname;
int radio_status=-1;
int radio_band=-1;
int volume=-1;
unsigned int radio_freq=0;
char freqstr[4]="";
char *radio_device=DEVICE;
int mixerfd;
int fd;
int interval = INT_LONG;
struct timeval interval_tv;
fd_set r_fds;
char *short_opts = "hnfv:s:c:d:";
struct option long_opts[] =
{
    {"help", 0, 0, 'h'},
    {"on", 0, 0, 'n'},
    {"off", 0, 0, 'f'},
    {"vol", 1, 0, 'v'},
    {"fm", 1, 0, 's'},
    {"am", 1, 0, 'c'},
    {"device", 1, 0, 'd'},
    {NULL, 0, 0, 0}
};


// Functions
void parse_command_line(int argc, char *argv[]);
void usage(void);
void version(void);
unsigned int check_freq(unsigned int freq, int band);
void check_volume(void);
void set_volume(void);
void make_freqstr(void);
void radio_on(void);
void radio_off(void);
void radio_tune(void);
unsigned int hundred_times_strtoint(char *str);


// Main
int main(int argc, char *argv[])
{
    myname = argv[0];
    if ( argc == 1 ) { 
        usage();
        exit(0);
    }
    parse_command_line(argc,argv);
    mixerfd = openmixer(0);
    //    addrecsrc(mixerfd, SOUND_MIXER_LINE);
    
    fd = radioinit(radio_device);
    if(radioprobe(fd))
    {
        radioclose(fd);
        fd = radioinit(radio_device);
        if(radioprobe(fd))
        {
            radio_status = OFF;
        }
    }
    
    if(radio_status == ON)
    {
        radio_on();
    }
    else if(radio_status == OFF)
    {
        radio_off();
    }

    if(radio_band == AM || radio_band == FM)
    {
	radio_freq = check_freq(radio_freq, radio_band);
	make_freqstr();
        radio_tune();
    }

    exit(0);
}

void parse_command_line(int argc, char *argv[])
{
    int opt_index = 0, c;
    char *opt_vol;
    char *opt_freq;
    
    while ((c = getopt_long(argc, argv, short_opts, long_opts, &opt_index)) != EOF) {
        switch( c ) {
            case 'h':
                usage();
                exit(0);
                break;
            case 'n':
                radio_status = ON;
                break;
            case 'f':
                radio_status = OFF;
                break;
            case 's':
                radio_band = FM;
                opt_freq = (char *)malloc(strlen(optarg)+1);
                strcpy(opt_freq, optarg);
                radio_freq = hundred_times_strtoint(opt_freq);
	        radio_freq = check_freq(radio_freq, radio_band);
                break;
            case 'c':
                radio_band = AM;
                opt_freq = (char *)malloc(strlen(optarg)+1);
                strcpy(opt_freq, optarg);
                radio_freq = atoi(opt_freq);
	        radio_freq = check_freq(radio_freq, radio_band);
                break;
	    case 'v':
                opt_vol = (char *)malloc(strlen(optarg)+1);
                strcpy(opt_vol, optarg);
		volume = atoi(opt_vol);
		check_volume();
                break;
            case 'd':
                radio_device = (char *)malloc(strlen(optarg)+1);
                strcpy(radio_device, optarg);
                break;
            default:
                exit(1);
        }
    }
}

void usage(void)
{
    version();
    printf("\n");
    printf("Usage: %s [OPTIONS]\n", myname);
    printf("\n");
    printf("  -n, --on                       "
           "turn radio on\n");
    printf("  -f, --off                      "
           "turn radio off\n");
    printf("  -s, --fm                       "
           "FM station to tune, ie 76.1\n");
    printf("  -c, --am                       "
           "AM channel to tune, ie 1134\n");
    printf("  -v, --vol                      "
           "set volume 0-100\n");
    printf("  -d, --device                   "
           "set device name, ie /dev/radio\n");
    printf("  -h, --help                     "
           "display this help and exit\n");
    printf("\n");
    printf("Author: Atsushi Umeshima <pcamfm@gmail.com>\n");
}

void version(void)
{
  printf("This is " NAME " " VERSION "\n");
  printf("Copyright (C) 2007 Atsushi Umeshima\n");
}

void radio_on()
{
    volume = getvolume(mixerfd);
    setvolume(mixerfd, 0);
    radioon(fd);
    usleep(ON_SLEEP_USEC);
    setvolume(mixerfd, volume);
}

void radio_off(void)
{
    volume = getvolume(mixerfd);
    setvolume(mixerfd, 0);
    radiooff(fd);
    usleep(OFF_SLEEP_USEC);
    setvolume(mixerfd, volume);
}

void radio_tune(void)
{
    volume = getvolume(mixerfd);
    setvolume(mixerfd, 0);
    radiotune(fd, freqstr, radio_band);
    usleep(TUNE_SLEEP_USEC);
    setvolume(mixerfd, volume);
}

void make_freqstr(void)
{
    if(radio_band == FM)
    {
        sprintf(freqstr, "%03d", FCODE(radio_freq));
#ifdef DEBUG
	printf("FM: %d, cmd: %s\n", radio_freq, freqstr);
#endif
    }
    else
    {
        sprintf(freqstr, "%03d", ACODE(radio_freq));
#ifdef DEBUG
	printf("AM: %d, cmd: %s\n", radio_freq, freqstr);
#endif
    }
}

unsigned int check_freq(unsigned int freq, int band)
{
    if (band == FM)
    {
        if(freq > FMFREQ_MAX)
        {
            freq = FMFREQ_MAX;
        }
        else if(freq < FMFREQ_MIN)
        {
            freq = FMFREQ_MIN;
        }
	freq -= freq % FMFREQ_STEP;
    }
    else
    {
        if(freq > AMFREQ_MAX)
        {
            freq = AMFREQ_MAX;
        }
        else if(freq < AMFREQ_MIN)
        {
            freq = AMFREQ_MIN;
        }
	freq -= freq % AMFREQ_STEP;
    }
    return freq;
}

void check_volume(void)
{
    if(volume > VOLUME_MAX)
    {
        volume = VOLUME_MAX;
    }
    else if(volume < VOLUME_MIN)
    {
        volume = VOLUME_MIN;
    }
}

void set_volume()
{
    check_volume();
#ifdef DEBUG
    printf("volume: %d\n", volume);
#endif
    setvolume(mixerfd, (int)volume);
}

